-----------------------------------------------------------------------------
-- |
-- Module      :  Generics.SYB.Builders
-- Copyright   :  (c) The University of Glasgow, CWI 2001--2004
-- License     :  BSD-style (see the LICENSE file)
-- 
-- Maintainer  :  generics@haskell.org
-- Stability   :  experimental
-- Portability :  non-portable (local universal quantification)
--
-- Convenience alias for "Data.Generics.Builders".
--
-----------------------------------------------------------------------------

module Generics.SYB.Builders (module Data.Generics.Builders) where

import Data.Generics.Builders
